float4   cnt    : register(c0);

struct VS_OUTPUT {
  float4 pos : POSITION;
  float4 tex : TEXCOORD0;
};

VS_OUTPUT vs_main(VS_OUTPUT input) {
  return input;
}

float iso(float3 p) {
  float  x = sin(p.x);
  float  y =    (p.y);
  float  z = sin(p.z);
  float3 ip = p * 24;
  float r = (cos(x) + cos(y) + cos(z));
  r -=  cos( cos(ip.x) * cos(ip.y) * cos(ip.z) ) * 0.1; //ekubo
  r -= sqrt( cos(ip.x) * cos(ip.y) * cos(ip.z) ) * 0.01; //melon
  return r;
}

//raycast
float raycast(float3 eye, float3 dir, float th) {
  float dt  = 15;
  float t = 0;
  for(int i = 0; i < 175 && dt > 0.001; i++) {
    float3 temp = eye + dir * t;
    if(iso(temp) < th) {
      t -= dt;
      dt *= 0.1;
    }
    t += dt;
  }
  return t;
}


float4 ps_main(float2 vp : vpos) : color {
  float2 uv = -1 + 2 * vp / (float2(800, 600));
  float time  = cnt.x;
  float flash = 1;//exp(1 - frac(time * 1.7));
  
  if(uv.y < -0.75 || uv.y > 0.75) return 0;
  float rotate = (time + sin(time * 0.5)) * (0.1);
  float3x3 mtx = {
    cos(rotate),  sin(rotate), 0,
    sin(rotate), -cos(rotate), 0,
    0,            0,           1
  };
  
  //rotate ray direction
  float3 dir    = normalize(
    float3(uv.x * 1.25 - sin(time * 3) * 0.01,
           uv.y        + cos(time * 4) * 0.01,
           1));
  dir = mul(mtx, dir).zxy;
  dir = mul(mtx, dir) / 100;
  
  //cast
  float3 eye    = float3(cos(time * 0.5) * 4, time * 2, sin(time * 0.5) * 4); //S + ~
  float  t      = raycast(eye, dir,  0.7);
  
  //calc normal.
  float3 ip     = eye + dir * t;
  float  iit    = iso(ip);
  float3 h      = float3(0.01, 0, 0);
  float3 n      = normalize(float3(
    iit - iso(ip + h.xyy),
    iit - iso(ip + h.yxy),
    iit - iso(ip + h.yyx)));
  
  //light.
  float3 light  = normalize(float3(1,  -0.5,  1));
  float3 normal = dot(light, n);
  
  //iro
  float4 col    = float4(dir + float3(2, 1, 0.5) * normal, 1) + t * 0.001;
  return col * (flash / 2);
  
}
